pub use ed25519_dalek::{Keypair, PublicKey, Signature, Signer};

use crate::prelude::*;

pub enum TrustLevel {
    Unknown,
    Distrusted,
    // Future extension
    // Transitively,
    Ultimately,
}

pub struct TrustDB {}

pub struct KeyExchangeInitiate {}

impl crate::msg::MessageParams for KeyExchangeInitiate {
    const NAME: &'static str = "KeyExchangeInitiate";
    const ENCRYPTED: bool = false;
}

pub enum KeyExchange {
    WaitForResponse(x25519_dalek::PublicKey),
    Completed(SymmetricKey),
}

#[derive(serde::Serialize, serde::Deserialize, Default, Debug)]
pub enum PacketHeader {
    #[default]
    NoCrypto,
    Signed(Signature),
    // u64 here is the nonce
    Encrypted(u64),
}

pub struct SymmetricKey {
    key: [u8; 16],
    nonce: [u8; 16],
}

#[derive(Default)]
pub(crate) struct SignPacket {}

impl Service for std::rc::Rc<SignPacket> {
    fn register_channels(&self, eroot: &mut EventRoot) {
        eroot
            .priority_channel::<fleck_core::SendPacketChannel>()
            .with::<order::Between<order::Postprocessing, order::Last>>()
            .subscribe(self, SignPacket::sign);
    }
}

impl SignPacket {
    fn sign(&self, api: &crate::Fleck, packet: &mut FleckEvent<FleckPacket>) {
        let msg = packet
            .data
            .msg
            .as_mut()
            .expect("packet got to signing stage without having a Msg?");

        let params = msg
            .saved_params
            .expect("packet got to signing stage without saved parameters?");

        // don't do anything if the packet doesn't want a signature
        if !params.signed {
            return;
        }

        api.services().with_service(|ns: &fleck_core::Nodes| {
            let keypair = ns.self_node().keypair().unwrap();
            let sign_data = bincode::serialize(msg).unwrap();
            msg.crypto_header = PacketHeader::Signed(keypair.sign(&sign_data));
        });
    }
}
