use std::{rc::Rc, collections::{HashMap, HashSet}, cell::{RefCell, Ref}};

use super::Service;

use crate::crypto::{Keypair, PublicKey};

pub struct InnerRef<'l, T> {
    r: Ref<'l, Option<T>>
}

impl<'l, T> InnerRef<'l, T> {
    fn build(from: Ref<'l, Option<T>>) -> Option<Self> {
        if from.is_none() {
            None
        }
        else {
            Some(Self {
                r: from
            })
        }
    }
}

impl<'l, T> AsRef<T> for InnerRef<'l, T> {
    fn as_ref(&self) -> &T {
        self.r.as_ref().unwrap()
    }
}

impl<'l, T> std::ops::Deref for InnerRef<'l, T> {
    type Target = T;
    fn deref(&self) -> &Self::Target {
        self.r.as_ref().unwrap()
    }
}

#[derive(Default)]
pub struct Node {
    addr: Option<std::net::SocketAddr>,
    pubkey: Option<PublicKey>,
    keypair: RefCell<Option<Keypair>>,
}

impl Node {
    pub fn pubkey(&self) -> Option<&PublicKey> {
        self.pubkey.as_ref()
    }
    pub(crate) fn keypair(&self) -> Option<InnerRef<'_, Keypair>> {
        InnerRef::build(self.keypair.borrow())
    }
}

pub struct Nodes {
    all_nodes: Vec<Rc<Node>>,
    self_node: Rc<Node>,
}

impl Default for Nodes {
    fn default() -> Self {
        let mut ret = Self {
            all_nodes: Default::default(),
            self_node: Default::default(),
        };

        ret.all_nodes.push(ret.self_node.clone());

        ret
    }
}

impl Nodes {
    pub fn self_node(&self) -> &Rc<Node> {
        &self.self_node
    }

    pub fn node_by_pubkey(&self, pubkey: PublicKey) -> Option<Rc<Node>> {
        None
    }
}

impl Service for Rc<Nodes> {}
